function [OutCellArray, Outconfig]=MapRotate(InCellArray, Inconfig)
%MapRotate rotates a map clockwise about the center of the map by a
% user-specified angle. Units on x and y axes must be same due to possible
% map boundary reshaping upon rotation. Note: Due to interpolation,
% multiple rotations can blur features in image. Maps with large aspect
% ratios suffer most.
%
%Inputs: Map
%Outputs: Map
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 8/13/10 - Version 1
% 11/16/2011 - Version 1.1 - Greg Smith - allow for non-rectangular arrays
% 6/4/2012 - Version 1.2 - Greg Smith - add option for data trimming
% 9/25/2012 - Version 1.3 - Greg Smith - add trimming option to keep data array unchanged

% use input as default output
Outconfig = Inconfig;
OutCellArray = InCellArray;

DegRot=Inconfig{strcmpi(Inconfig, 'Rotation (Degrees)'),2};
TrimData=Inconfig{strcmpi(Inconfig, 'Trim Data'),2};

validinputs=0;

%Get inputs
while ~validinputs
    answers = inputdlg({'Rotation [degrees about center]','Trim Data?  [2 = keep same size,  1 = trim,  0 = no trim]'},'MapRotate',1,{DegRot,TrimData});
    if ~isempty(answers)
        % get rotation
        tempdeg=answers{1};              
        tempdeg=str2double(tempdeg);
        if ~isempty(tempdeg)&&isnumeric(tempdeg)
            validinputs=1;             
            DegRot=tempdeg;
        else
            msgbox('Please enter valid rotation','Invalid Inputs','warn','modal')
        end
        
        % get trimming option
        trim = answers{2};
        trim = str2num(trim);   %#ok - G.S. - 6/4/2012 - need str2num since answer could be any form of boolean
        if ~isempty(trim)
            validinputs=1;             
            TrimData=trim;
        else
            msgbox('Please enter valid trimming option','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'Rotation (Degrees)'),2}=num2str(DegRot);
Outconfig{strcmpi(Outconfig, 'Trim Data'),2}=num2str(TrimData);

%rotation by 90 is simple - use the fast method
if abs(mod(DegRot,90)) < 10^-7 || abs(90 - abs(mod(DegRot,90))) < 10^-7
    % flip data
    OutCellArray{1}.data=rot90(InCellArray{1}.data,round(-DegRot/90));
    
    % reverse size information
    xSize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
    ySize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});
    OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'X Size'),2} = num2str(ySize);
    OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'Y Size'),2} = num2str(xSize);
    return
end

% convert to radians
RadRot=DegRot*pi/180;

% extract data, header, dimensions, and min value
% because of reshaping with rotation, x and y units must be same
xSize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
ySize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});
xUnit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yUnit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
if (~strcmpi(xUnit,yUnit))
    msgbox('Axes must have the same units. Please convert and try again. Calculation is aborting...','Invalid Inputs','warn','modal');
    Outconfig=Inconfig;
    OutCellArray=InCellArray;
    return
end
surfmap = InCellArray{1}.data;
[pointsY,pointsX] = size(surfmap);
minVal = min(min(surfmap));

% convert data to polar coordinates
x=linspace(-xSize/2, xSize/2, pointsX);
y=linspace(-ySize/2, ySize/2, pointsY);
[x,y]=meshgrid(x,y);
[theta, rho, z] = cart2pol(x,y,surfmap);

% rotate map
theta = theta+RadRot;

% convert to Cartesian coordinates and interpolate to new grid
[xnew,ynew,znew]=pol2cart(theta(:), rho(:), z(:));
F = TriScatteredInterp(xnew, ynew, znew,'linear');

% create new output grid
% find new bounding box extents by rotating existing bounding box
switch TrimData
    case 2
        % keep data same size as original
        maxXRot=xSize/2;
        minXRot=-xSize/2; 
        maxYRot=ySize/2;
        minYRot=-ySize/2;
    case 1
        % trim to smallest size
        maxXRot=max(xnew(znew>=minVal)); % extract min/max values of new bounding box - eliminate all points with NaN
        minXRot=min(xnew(znew>=minVal)); 
        maxYRot=max(ynew(znew>=minVal));
        minYRot=min(ynew(znew>=minVal));
    otherwise
        % do not trim
        avg = (1+sqrt(2))/2;
        amp = (sqrt(2)-1)/2;
        maxXRot=xSize/2*(avg-amp*cos(4*RadRot)); % calculate max and min of new box - do not trim NaNs
        minXRot=-xSize/2*(avg-amp*cos(4*RadRot)); 
        maxYRot=ySize/2*(avg-amp*cos(4*RadRot));
        minYRot=-ySize/2*(avg-amp*cos(4*RadRot));
end
xRot=linspace(minXRot, maxXRot, ceil((maxXRot-minXRot)/xSize*pointsX));
yRot=linspace(minYRot, maxYRot, ceil((maxYRot-minYRot)/ySize*pointsY));
[xRot,yRot]=meshgrid(xRot,yRot);

% evaluate interpolation grid over new bounding box
surfmap=F(xRot,yRot);

% update parameters for return
OutCellArray{1}.data=surfmap;
OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'X Size'),2} = num2str(length(xRot)/pointsX*xSize);
OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'Y Size'),2} = num2str(length(yRot)/pointsY*ySize);

% Below is an alternative scale factor which is more accurate when multiple
% rotations are sequentially applied, but may lead to changing "size" of
% trimmed circular images due to numerical round-off that occurs during
% rotation.
%OutCellArray{1}.header{strcmpi(OutCellArray{1}.header, 'X Size'),2} = num2str(maxXRot-minXRot);
end