function [OutCellArray, Outconfig]=ZernikeTermRemoval(InCellArray, Inconfig)
%ZernikeTermRemoval removes zernike terms from a map, by subtracting
%polynomials generated by recursively.  Can handle both annular and
%standard zernike removal.
%
%Inputs: Map
%Outputs: Map
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
%10/30/11 - Version 3
%11/14/11 - Version 3.1


Outconfig=Inconfig;

OutCellArray{1}=InCellArray{1};

dmap=InCellArray{1}.data;



ZernRemove=Inconfig{strcmpi(Inconfig, 'Terms Removed'),2};

validinputs=0;

%Get inptuts
while ~validinputs
    answers = inputdlg({'Terms to Remove (ex: 1,2,3,4)'},'ZernikeTermRemoval',1,{ZernRemove});
    if ~isempty(answers) %parse answers
        tempterms=answers{1};      
        expr='(\d)*,|(\d)*';
        tempscell=regexp(tempterms,expr,'tokens'); %get terms from CSL
        tempterms=[];
        for kk=1:length(tempscell)
            temp=floor(str2num(char(tempscell{kk})));
            if ~isempty(temp)&&isnumeric(temp)
                tempterms(end+1)=temp;
            end
        end

        if isempty(tempterms) %check validity
            validinputs=1;             
            ZernRemove=[];
        elseif isnumeric(tempterms)
            validinputs=1;            
            ZernRemove=tempterms;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

outstring=sprintf('%i, ', ZernRemove);

%modify config
Outconfig{strcmpi(Outconfig, 'Terms Removed'),2}=outstring(1:end-2);





ztypedefault=Inconfig{strcmpi(Inconfig, 'Z Type'),2};

ztype=questdlg('Select Zernike Type:','Map2ZernikeRMS','Annular','Standard',ztypedefault);

Outconfig{strcmpi(Outconfig, 'Z Type'),2}=ztype;


switch ztype
    case 'Standard'       
        epsilon=0;
    case 'Annular'        
        obsdefault=Inconfig{strcmpi(Inconfig, 'Obscuration'),2};
        
        obsdefault=questdlg('Select Obscuration Ratio Method:','Map2ZernikeRMS','Auto','Manual',obsdefault);
        
        Outconfig{strcmpi(Outconfig, 'Obscuration'),2}=obsdefault;
        
        
        switch obsdefault
            case 'Auto'
                epsilon=ObscurationFinder(dmap);
            case 'Manual'
                epsilon=Inconfig{strcmpi(Inconfig, 'Epsilon'),2};
                
                validinputs=0;
                
                %Get user input
                while ~validinputs
                    answers = inputdlg({'Obscuration Ratio:'},'Map2ZernikeRMS',1,{epsilon});
                    if ~isempty(answers)
                        tempobs=answers{:};
                        tempobs=str2num(tempobs);
                        if ~isempty(tempobs)
                            validinputs=1;
                            epsilon=tempobs;
                        else
                            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
                        end
                    else
                        OutCellArray={};
                        return; %If answers is empty, user canceled
                    end
                end
                
                Outconfig{strcmpi(Outconfig, 'Epsilon'),2}=num2str(epsilon);                                
        end
end




x=linspace(-1, 1, size(dmap,2));
y=linspace(-1, 1, size(dmap,1));

[x,y]=meshgrid(x,-1*y');

[theta, rho] = cart2pol(x,y);

for ii=1:length(theta(:))
    if theta(ii)>pi
        theta(ii)=theta(ii)-2*pi;
    end
end



OutCellArray{1}.data=RemoveZernikes(rho, theta, max(ZernRemove), epsilon, dmap, ZernRemove);


%     function wm=WeightMapper(dmap)
%         wm=ones(size(dmap,1),size(dmap,2));
%         
%         wm(isnan(dmap))=0;
%         
%         wm=conv2(wm,[1,1,1;1,0,1;1,1,1],'same');
%         
%         wm(wm<=2)=0;
%         wm(wm<6&wm>2)=.5;
%         wm(wm>=6)=1;
%         
%         wm(isnan(dmap))=0;
%         
%         %wm=ones(size(dmap,1),size(dmap,2));
%     end


    function [epsilon] = ObscurationFinder(dmap)
        %UNTITLED Summary of this function goes here
        %   Detailed explanation goes here
        
        bwmap=dmap;
        
        bwmap(isnan(dmap))=0;
        bwmap(~isnan(dmap))=1;
        
        closemap=imfill(bwmap);
        
        holemap=xor(closemap,bwmap);
        
        x=linspace(-1, 1, size(dmap,2));
        y=linspace(-1, 1, size(dmap,1));
        
        [x,y]=meshgrid(x,-1*y');
        
        [~, rho] = cart2pol(x,y);
        
        rho(holemap)=NaN;
        
        rho=rho(:);
        rho(isnan(rho))=[];
        
        rho=unique(rho);
        
        epsilon=rho(1)*(1-1/size(dmap,1)); %correction for overestimation by a pixel
    end




    function dmap=RemoveZernikes(rho, theta, maxTerm, epsilon, dmap, ZernRemove)
        m=0;
        n=0;
        
        tt=1;
        indices(1,:)=[0,0,0];
        
        while tt<maxTerm
            if m==n||m==-1*n
                n=n+1;
                if mod(n,2)==0, m=0; else m=1; end
            else
                m=m+2;
            end
            if m==0
                indices(tt+1,:)=[m, n, (n-abs(m))/2]; %We can't index to 0, so zernike z0 maps to 1
            else
                if mod(tt,2)~=0 %Strangely enough, ZEMAX uses this to decide the order of terms
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                else
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                end
            end
            
            tt=tt+1;
        end
        
        indices=indices(1:maxTerm,:);
        
        if size(indices,1)>1
            Maxs=max(abs(indices));
            MaxM=Maxs(1);
            MaxJ=Maxs(3);
        else
            MaxM=abs(indices(1));
            MaxJ=abs(indices(3));
        end
        
        ThisRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        LastRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        
        thisH=zeros(1,MaxM+MaxJ-1);
        lastH=zeros(1,MaxM+MaxJ-1);
                
        
        zc=cell(1,MaxM+MaxJ-1);
        
        rho=rho(:);
        theta=theta(:);
        tdmap=dmap(:);
        %weightmap=weightmap(:);
        
        
        rho(end+1)=0;
        theta(end+1)=0;
        
        
        X=(2.*((rho.^2-epsilon^2)./(1-epsilon^2)))-1;
        X(isnan(rho))=NaN;
        
        for mm=0:MaxM
            Qsum=zeros(size(rho));
            for jj=0:MaxM+MaxJ-1-(mm-1)
                if mm==0
                    if jj==0
                        TempQ=ones(size(rho));
                        TempQ(isnan(rho))=NaN;
                        ThisRow{jj+1}=TempQ;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    elseif jj==1
                        ThisRow{jj+1}=X;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    else
                        ThisRow{jj+1}=(2*(jj-1)+1).*X.*ThisRow{jj}-(jj-1).*ThisRow{jj-1};
                        ThisRow{jj+1}=ThisRow{jj+1}./((jj-1)+1);
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    end
                else
                    a=2*(2*jj+2*mm-1)*lastH(jj+1);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    c=a/b;
                    
                    Qsum=Qsum+LastRow{jj+1}.*LastRow{jj+1}(end)./lastH(jj+1);
                    
                    ThisRow{jj+1}=c.*Qsum;
                    
                    a=2*(2*jj+2*mm-1)*LastRow{jj+2}(end);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    
                    thisH(jj+1)=-1*(a/b)*lastH(jj+1);
                end
            end
            
            LastRow=ThisRow;
            lastH=thisH;
            
            rhoM=rho.^mm;
            
            for ll=1:size(indices,1)
                if abs(indices(ll,1))==mm
                    m=indices(ll,1);
                    n=indices(ll,2);
                    j=indices(ll,3);
                    
                    scale=(1-epsilon^2)/(2*(2*j+mm+1)*thisH(j+1));
                    scale=scale^.5;
                    scale=scale.*rhoM;
                    
                    if m==0 %Multiply by cos or sin if nessicary, and normalize so RMS=1
                        zernike=sqrt(n+1).*scale.*ThisRow{j+1};
                    elseif m>0
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*cos(mm.*theta);
                    else
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*sin(mm.*theta);
                    end
                    
                    
                    zernike(end)=[];
                    
                    zernike(isnan(tdmap))=NaN;
                    
                    zc{ll}=zernike;
                    
                    
                end
            end
        end
        
        
        function zt=proj(z1,z2) 
            term1 = z1.*z2;
            term1 = sum(sum(term1(~isnan(term1))));
            term2 = z2.*z2;
            term2 = sum(sum(term2(~isnan(term2))));
            zt=term1/term2;
            %zt=(nansum(nansum(z1.*z2))/nansum(nansum(z2.*z2)));
        end
        
        %Gram-Schmidt
        cmat=zeros(length(zc));
        
        tzc=zc;
        
        for kk=1:length(zc)   
            cmat(kk,kk)=1;
            
            for jj=1:kk-1
                B=proj(zc{kk},zc{jj});
                
                zc{kk}=zc{kk}-B*zc{jj};
                
                cmat(jj,kk)=-1*B;
            end
            
            tmap=zc{kk}.*tdmap;
            tmap(isnan(tmap))=0;
            
            %a=trapz(trapz(tmap));
            a=sum(sum(tmap));
            
            
            
            z2=zc{kk}.^2;
            z2(isnan(z2))=0;
            
            %b=trapz(trapz(z2));
            b=sum(sum(z2));
            
            z=a/b;            
            
            tdmap=tdmap-z.*zc{kk};
            
            zout(kk)=z;
        end
        
        zout=cmat*zout';
        
        for kk=1:length(zc)
            if any(kk==ZernRemove)
                dmap=dmap-zout(kk).*reshape(tzc{kk},size(dmap,1),size(dmap,2));
            end
        end
    end
end

